from random import randint

from django.db import models


class QuoteManager(models.Manager):
    def get_random(self, **kwargs):
        """
        Returns a 'random' (optionally filtered) quote.

        This method of retreiving a random record is used instead of Django's
        order_by('?') because of how slow that can be for some databases.

        """
        filters = {'public': True}
        filters.update(**kwargs)
        quotes = self.filter(**filters)


        num_quotes = quotes.count()
        try:
            random_offset = randint(0, num_quotes - 1)
            quote = quotes[random_offset]
        except ValueError:
            quote = self.none()
        return quote


class Quote(models.Model):
    """
    Quotes and their sources.

    """
    id = models.AutoField(primary_key=True, db_column='quote_id')
    source = models.CharField(max_length=250)
    content = models.TextField()
    detailed_citation = models.CharField(max_length=500, default='', blank=True)
    public = models.BooleanField(default=False)
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    objects = QuoteManager()

    def __unicode__(self):
        return self.content



from django.contrib import admin

from quotidian import models


class QuoteAdmin(admin.ModelAdmin):
    list_display = ('content', 'source', 'public')
    list_filter = ('public',)
    search_fields = ('content',)


# Register models
admin.site.register(models.Quote, QuoteAdmin)



from django.test import TestCase

from quotidian.models import Quote

class QuoteModelTests(TestCase):
    """
    Tests for the Quote model and manager.

    """
    fixtures = ['test_data.json']

    def test_random_public_only(self):
        """
        Only quotes marked as public should be returned by get_random.

        """
        # Delete all public quotes
        public_quotes = Quote.objects.filter(public=True)
        public_quotes.delete()
        # Insert a non-public quote so we know there's at least one stored
        Quote.objects.create(source='Test Source',
                             content='Test Content',
                             public=False)
        # None of the quotes should be public, so this should be empty
        self.assertEqual(len(Quote.objects.get_random()), 0)

    def test_kwarg_filters(self):
        """
        Tests additional filters for Quote.objects.get_random().
        
        """
        filter_detail = Quote.objects.get_random(detailed_citation='Detail3')
        filter_source = Quote.objects.get_random(source='Source3')
        self.assertEqual(filter_detail, filter_source)

        filter_date = Quote.objects.get_random(public=False,
                                               created__gte='2011-08-23 15:08:50',
                                               created__lte='2011-08-23 15:08:59')
        filter_content = Quote.objects.get_random(public=False,
                                                    content='Content2')
        self.assertEqual(filter_date, filter_content)



from quotidian.models import Quote


def random_public_quote(request):
    """
    Returns context dict containing a single quote.

    This is chosen from amongst quotes flagged as public.

    """
    quote = Quote.objects.get_random()
    return {'quotidian_quote': quote}
